'use strict';

module.exports = {
  SET_TOPIC: 'squelch-client-state/SET_TOPIC',

  /**
   * Action creator for setting the topic
   */
  setTopic: function setTopic(_ref) {
    var id = _ref.id,
        chan = _ref.chan,
        topic = _ref.topic;

    return {
      id: id,
      type: module.exports.SET_TOPIC,
      chan: chan,
      topic: topic
    };
  },


  SET_TOPIC_WHO: 'squelch-client-state/SET_TOPIC_WHO',

  /**
   * Action creator for setting the topic who and time
   */
  setTopicWho: function setTopicWho(_ref2) {
    var id = _ref2.id,
        chan = _ref2.chan,
        hostmask = _ref2.hostmask,
        time = _ref2.time;

    return {
      id: id,
      type: module.exports.SET_TOPIC_WHO,
      chan: chan,
      hostmask: hostmask,
      time: time
    };
  },


  UPDATE_NAMES: 'squelch-client-state/UPDATE_NAMES',

  /**
   * Action creator for updating channel users from NAMES reply.
   */
  updateNames: function updateNames(_ref3) {
    var id = _ref3.id,
        chan = _ref3.chan,
        names = _ref3.names;

    return {
      id: id,
      type: module.exports.UPDATE_NAMES,
      chan: chan,
      names: names
    };
  },


  USER_JOIN: 'squelch-client-state/USER_JOIN',

  /**
   * Action creator for a user joining a channel.
   */
  userJoin: function userJoin(_ref4) {
    var id = _ref4.id,
        chan = _ref4.chan,
        nick = _ref4.nick,
        me = _ref4.me;

    return {
      id: id,
      type: module.exports.USER_JOIN,
      chan: chan,
      nick: nick,
      me: me
    };
  },


  USER_LEAVE: 'squelch-client-state/USER_LEAVE',

  /**
   * Action creator for a user leaving a channel.
   */
  userLeave: function userLeave(_ref5) {
    var id = _ref5.id,
        chan = _ref5.chan,
        nick = _ref5.nick,
        me = _ref5.me;

    return {
      id: id,
      type: module.exports.USER_LEAVE,
      chan: chan,
      nick: nick,
      me: me
    };
  },


  USER_QUIT: 'squelch-client-state/USER_QUIT',

  /**
   * Action creator for a user quitting a channel.
   */
  userQuit: function userQuit(_ref6) {
    var id = _ref6.id,
        nick = _ref6.nick,
        channels = _ref6.channels;

    return {
      id: id,
      type: module.exports.USER_QUIT,
      nick: nick,
      channels: channels
    };
  },


  ADD_CHANNEL_MODE: 'squelch-client-state/ADD_CHANNEL_MODE',

  /**
   * Action creator for adding a mode to a channel.
   * @param {Object} state The current state
   * @param {Object} e The +mode event object from squelch-client
   */
  addChannelMode: function addChannelMode(state, _ref7) {
    var id = _ref7.id,
        chan = _ref7.chan,
        mode = _ref7.mode,
        param = _ref7.param;

    return {
      id: id,
      type: module.exports.ADD_CHANNEL_MODE,
      prefix: state.iSupport['PREFIX'][mode] || undefined,
      chan: chan,
      mode: mode,
      param: param
    };
  },


  REMOVE_CHANNEL_MODE: 'squelch-client-state/REMOVE_CHANNEL_MODE',

  /**
   * Action creator for removing a mode to a channel.
   * @param {Object} state The current state
   * @param {Object} e The -mode event object from squelch-client
   */
  removeChannelMode: function removeChannelMode(state, _ref8) {
    var id = _ref8.id,
        chan = _ref8.chan,
        mode = _ref8.mode,
        param = _ref8.param;

    return {
      id: id,
      type: module.exports.REMOVE_CHANNEL_MODE,
      prefix: state.iSupport['PREFIX'][mode] || undefined,
      chan: chan,
      mode: mode,
      param: param
    };
  },


  REMOVE_CHANNEL: 'squelch-client-state/REMOVE_CHANNEL',

  /**
   * Action creator for removing the data for a channel.
   */
  removeChannel: function removeChannel(_ref9) {
    var id = _ref9.id,
        chan = _ref9.chan;

    return {
      id: id,
      type: module.exports.REMOVE_CHANNEL,
      chan: chan
    };
  },


  SET_ISUPPORT: 'squelch-client-state/SET_ISUPPORT',

  /**
   * Action creator for a updating iSupport values.
   * Accepts parsed reply from squelch-client raw event
   */
  setISupport: function setISupport(_ref10) {
    var id = _ref10.id,
        params = _ref10.params;

    var iSupport = {};
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = params.slice(1)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var item = _step.value;

        // Skip the last param, which is usually something like
        // ":is supported by this server"
        if (item.indexOf(' ') !== -1) break;

        var split = item.split('=');
        if (split.length === 1) {
          iSupport[item] = true;
        } else if (split[0] === 'PREFIX') {
          iSupport['PREFIX'] = {};
          var match = /\((.+)\)(.+)/.exec(split[1]);
          for (var i = 0; i < match[1].length; i++) {
            iSupport['PREFIX'][match[1][i]] = match[2][i];
          }
        } else if (split[0] === 'CHANMODES') {
          iSupport['CHANMODES'] = split[1].split(',');
          // CHANMODES[0,1] always require param
          // CHANMODES[2] requires param on set
          // CHANMODES[3] never require param
        } else {
          iSupport[split[0]] = split[1];
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return {
      id: id,
      type: module.exports.SET_ISUPPORT,
      iSupport: iSupport
    };
  },


  SET_CONNECTED: 'squelch-client-state/SET_CONNECTED',

  /**
   * Action creator for setting the connected status.
   */
  setConnected: function setConnected(_ref11) {
    var id = _ref11.id,
        connected = _ref11.connected;

    return {
      id: id,
      type: module.exports.SET_CONNECTED,
      connected: connected
    };
  },


  SET_CONNECTING: 'squelch-client-state/SET_CONNECTING',

  /**
   * Action creator for setting the connecting status.
   */
  setConnecting: function setConnecting(_ref12) {
    var id = _ref12.id,
        connecting = _ref12.connecting;

    return {
      id: id,
      type: module.exports.SET_CONNECTING,
      connecting: connecting
    };
  },


  CHANGE_NICK: 'squelch-client-state/CHANGE_NICK',

  changeNick: function changeNick(_ref13) {
    var id = _ref13.id,
        oldNick = _ref13.oldNick,
        newNick = _ref13.newNick;

    return {
      id: id,
      type: module.exports.CHANGE_NICK,
      oldNick: oldNick,
      newNick: newNick
    };
  },


  DISCONNECT: 'squelch-client-state/DISCONNECT',

  disconnect: function disconnect(_ref14) {
    var id = _ref14.id;

    return {
      id: id,
      type: module.exports.DISCONNECT
    };
  }
};